#include <descrip.h>
#include <syidef.h>
#include <atrdef.h>
#include <fibdef.h>
#include <iodef.h>
#include <fchdef.h>
#include <rms.h>

#define ERR(s) !((s) & 1)	/* VMS system error */

#ifndef SYI$_VERSION
#define SYI$_VERSION 4096	/* VMS 5.4 definition */
#endif

struct EB_header    /* Common header of extra block */
{   ush	tag;
    ush	size;
    uch data[1];
};

/*------ Old style INFO-ZIP extra field definitions -----*/

#ifndef VAXC
				/* This definition may be missed */
struct XAB {
    unsigned char xab$b_cod;
    unsigned char xab$b_bln;
    short int xabdef$$_fill_1;
    char *xab$l_nxt;
};

#endif

#define BC_MASK 	07	/* 3 bits for compression type */
#define BC_STORED	0	/* Stored */
#define BC_00		1	/* 0byte -> 0bit compression */
#define BC_DEFL		2	/* Deflated */

/*
*   Extra record format
*   ===================
*   signature       (2 bytes)   = 'I','M'
*   size            (2 bytes)
*   block signature (4 bytes)
*   flags           (2 bytes)
*   uncomprssed size(2 bytes)
*   reserved        (4 bytes)
*   data            ((size-12) bytes)
*   ....
*/

struct IZ_block
{
    ush sig;			/* Extra field block header structure */
    ush size;
    ulg bid;
    ush flags;
    ush length;
    ulg reserved;
    uch body[1];
};

/*
 *   Extra field signature and block signatures
 */

#define IZ_SIGNATURE "IM"
#define FABL    (cc$rms_fab.fab$b_bln)
#define RABL    (cc$rms_rab.rab$b_bln)
#define XALLL   (cc$rms_xaball.xab$b_bln)
#define XDATL   (cc$rms_xabdat.xab$b_bln)
#define XFHCL   (cc$rms_xabfhc.xab$b_bln)
#define XKEYL   (cc$rms_xabkey.xab$b_bln)
#define XPROL   (cc$rms_xabpro.xab$b_bln)
#define XRDTL   (cc$rms_xabrdt.xab$b_bln)
#define XSUML   (cc$rms_xabsum.xab$b_bln)
#define EXTBSL  4		/* Block signature length   */
#define RESL    8		/* Rserved 8 bytes  */
#define EXTHL   (4+EXTBSL)
#define FABSIG  "VFAB"
#define XALLSIG "VALL"
#define XFHCSIG "VFHC"
#define XDATSIG "VDAT"
#define XRDTSIG "VRDT"
#define XPROSIG "VPRO"
#define XKEYSIG "VKEY"
#define XNAMSIG "VNAM"
#define VERSIG  "VMSV"

typedef unsigned char byte;

struct iosb
{   ush	    status;
    ush	    count;
    ulg	    spec;
};

/*------------ PKWARE extra block definitions ----------*/

/* Structure of PKWARE extra header */

#ifdef VMS_ZIP

struct PK_info
{
    ush tag_ra;	ush len_ra;	byte ra[ATR$S_RECATTR];
    ush tag_uc;	ush len_uc;	byte uc[ATR$S_UCHAR];
    ush tag_jr;	ush len_jr;	byte jr[ATR$S_JOURNAL];
    ush tag_cd;	ush len_cd;	byte cd[ATR$S_CREDATE];
    ush tag_rd;	ush len_rd;	byte rd[ATR$S_REVDATE];
    ush tag_ed;	ush len_ed;	byte ed[ATR$S_EXPDATE];
    ush tag_bd;	ush len_bd;	byte bd[ATR$S_BAKDATE];
    ush tag_rn;	ush len_rn;	ush  rn;
    ush tag_ui;	ush len_ui;	byte ui[ATR$S_UIC];
    ush tag_fp;	ush len_fp;	byte fp[ATR$S_FPRO];
    ush tag_rp;	ush len_rp;	byte rp[ATR$S_RPRO];
};

#endif	/* ?VMS_ZIP */

/* PKWARE "VMS" tag */
#define	PK_SIGNATURE	    0x000C
#define IZ_NEW_SIGNATURE    0x010C  /* New signature, extra record format
				    *  mostly compatible with PKWARE's */

/* Total number of attributes to be saved */
#define VMS_ATTR_COUNT	11
#define VMS_MAX_ATRCNT	20

struct PK_field
{   ush		tag;
    ush		size;
    byte	value[1];
};

#define PK_FLDHDR_SIZE	4

struct PK_header
{   ush	tag;
    ush	size;
    ulg	crc32;
    byte data[1];
};

#define	PK_HEADER_SIZE	8

#ifdef	VMS_ZIP
/* File description structure for Zip low level I/O */
struct ioctx
{   ush			chan;
    int			status;
    struct iosb		iosb;
    long		vbn;
    long		size;
    long		rest;
    struct PK_info	PKi;
    long		acllen;
    uch			aclbuf[ATR$S_READACL];
};
#endif	/* VMS_ZIP */
